// the following routines are from my standard library

				  /* standard character string lengths */
#define USELEN        80              /* usual line length */
#define STRLEN        USELEN + 2      /* plus room for terminator(s) */
#define USEBIG        160             /* two lines */
#define BIGSTR        USEBIG + 2      /* ditto */
#define USELRG        255             /* for big jobs */
#define LRGSTR        USELRG + 1      /* ditto */
#define USEHUGE       511             /* for REALLY big jobs */
#define HUGESTR       USEHUGE + 1     /* ditto */
#define NUMBERSIZE    40              /* for numeric input entry dialogs */

#define MYPMASSERT
#ifdef MYPMASSERT
  #define pmassert( hab, exp )\
  {\
    if( !( exp ) ) \
    { char ebuff[ 128 ]; unsigned long errorid; unsigned short rc;\
      errorid = WinGetLastError( hab ); \
      sprintf( ebuff, "Line %d\nFile %s\nLast Error %#x\nExpression %s\n",\
	       __LINE__, __FILE__, errorid, #exp );\
      rc = WinMessageBox( HWND_DESKTOP, HWND_DESKTOP, ebuff,\
			  "Assertion failed. Continue?", 0, MB_YESNO  );\
      if( rc == MBID_NO ) exit( 1 ); }\
  }
#else
  #define pmassert( hab, exp )
#endif


BOOL CenterDlg( HWND hwndDlg, HWND hwndObj );
// Center Object on Window.  Return TRUE if successful, FALSE if not

BOOL CreateMessageWindow( void );
// Creates a window thread for displaying information, such
// as tracking changing values without having to press a button
// or interupting program flow. Must call 'DestroyMessageWindow'
// to close window before application termination.
// After creating window, use 'InfoMessage' with 'flStyle'==0
// to display information. in it.  So far, no formating is
// performed.
// Returns TRUE if creation was succesful, or FALSE if the
// window is already open or an error occured.

BOOL DestroyMessageWindow( void );
// Destroys the window thread created by 'CreateMessageWindow'
// Returns TRUE if succesful, or FALSE if not thread was not
// running, or an error occured.

void DisplayError( HWND hwnd, char *szTitle, char *szSource );
// Displays a message box with details on the last error
// that occured in window 'hwnd'.  The message displayed is:
//      szTitle
//      Source: 'szSource'
//      Message: 'system error message'
//      Severity: 'severity message'

short ErrorMessage( char *szMessage );
// Display a message box titled "Error" with a hand icon and
// displaying 'szMessage'.  Returns the code for the button
// selected (MBID_OK or MBID_CANCEL), or MBID_ERROR if the
// window creation was not successful.

long GetLastError( HAB hab, char *szMessage );
// Returns the severity code for the last error that occured in
// process 'hab'.  The string 'szMessage' is filled with:
// "Sys Info:\nError no 0xdddd" where 0xddd is the error id.

short InfoMessage( char *szTitle, long flStyle, char *szFmt,...);
// Display a window with title 'szTitle' and a message that is
// formated using sprinf, 'szFmt' and any addditional parameters
// This is a handy messaging service with three possibilities:
//
// 1) if 'flStyle' contains ORed window style flags MB_* (!=0),
//   a message box is displayed with the appropriate properites.
//   Returned is the MBID_* code for the button pushed. (This is
//   good for occasional info messages and minor problems).
//
// 2) if 'flStyle'==0 && no 'CreateMessageWindow'  (above);
//   the default MB_OKCANCEL is used to create "OK' and "CANCEL'
//   buttons.  If the user selects 'CANCEL',' the program is
//   terminated (great for severe debugging,)
//   MBID_OK is returned if the 'OK' button pushed (use at
//   own risk!
//
//.3) if 'flStyle'==0 && a 'CreateMessageWindow' exists (above);
//   The message is displayed as a single line (\n later?) and
//   MBID_OK is always returned.  Handy for situations where
//   redraws are frequent and you don't know what the f*** is
//   going on,

void *myfree( void *pMemory );
// frees memory pointed to by 'pMemory', and returns NULL

BOOL SetNumberSpin( HWND hwnd, MPARAM mp1, long lItemIDD, long lSelect,
		    long lLower, long lUpper );
// Sets up a numerical spin button.  After rudimentary checking
// sets the 'lLower' and 'lUpper' limits and set the selection
// to 'lSelect'.

BOOL SetStaticText( HWND hwnd, MPARAM mp1, long lItemIDD, long lSize,
		char *szFmt, ... );
// Replace the contents of static text control 'lItemIDD' in
// dialog 'hwnd' with a text string of size lSize that is
// formated according to szFmt and follow-on arguments.
// Returns TRUE if successful, or FALSE for an error or mp1
// is not for lItemIDD

void *stralloc( long lAmount, char *szPlace );
// Allocated 'lAmount' of memory from the heap.  If the
// operation was not successful, an error message is displayed:
// "Could not allocate 'lAmount bytes for 'szPlace'"
// (Note this could be menaingless, since the error message
// tries to get memory, so if none is available ...)
// Returns a pointer of type void to the memory block, or
// NULL if not successful.

char *substrcpy( char *dest, char *source, int start, int amt );
// copy 'amt' chatacters starting at 'start' from 'source' to
// 'dest'  If 'start' is larger than strlen( source ), then
// no characters are copied.  If ( start + amt ) is larger than
// strlen( source ), then only the remainder of 'source'
// starting at start is copied.  'dest' must point to a string
// array that at least ( amt + 1 ) long.
// Returned is a pointer to 'dest'

BOOL SysErrorMessage( HAB hab, char *szTitle, char *szSource );
// Displays a message box with details on the last error
// that occured in thread 'hab'.  The message displayed is:
//      szTitle
//      Source: 'szSource'
//      Message: 'system error message'
//      Severity: 'severity message'
//
// Returns TRUE if the error is recoverable and the user
// presses the OK button, FALSE if not.

