/* Displays a calander icon for the specified month on the Desktop
   Converted from the REXX script WPCalendar.cmd created by:
   Frank Wochatz <mail@subsys.de>
   Berlin, 03. Oct 2005
*/
#define INCL_PM		// PM library
#include <os2.h>

#include <dos.h>	// C library
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "wpcal.h"	// this module

// standard working string lengths
#define BIGSTR	256
#define HUGESTR 512
#define STRLEN	82

// for convenience I made these global variables for module mycal
static long weekstart = 1;	// 0 = Monday, 1= Sunday
static char *language = "en";	// supported: "de", "en", "fr", "nl"

HOBJECT WPCalendar( HAB hab, struct date ShowDate, long lStart,
		    char* path, char* lang )
{
  char dayicon[HUGESTR], objecttext[HUGESTR], setup[HUGESTR];
  HOBJECT hIcon;

  // initialize variables, making sure they are within bounds
  weekstart = min( 1, max( 0, lStart) );
  substrcpy( language, lang, 0, 2 );

  // a little safety check
  ShowDate.da_mon = max( 1, min( ShowDate.da_mon, 12 ));
  ShowDate.da_day = max( 1, min( ShowDate.da_day,
		TotalDaysOfMonth(  ShowDate.da_mon, ShowDate.da_year ) ) );
  // leave the years alone, even though I am not sure what happens with
  // negative (BC) or non-recent years

  // create the icon title text
  BuildCalendar( (long) ShowDate.da_mon, (long) ShowDate.da_year, objecttext );

  //  WinCreateObject can only handle title strings that are less than 256
  //  characters long, so truncate to be safe.  I'm not sure if this a BC++ 2.0
  //  or a OS/2 2.1 limitation,  (Borland stopped updating before Warp 3 was
  //  released.)
  if ( strlen( objecttext ) > 255 )
    objecttext[255] = '\0';

  // create the day icon file path, choosing red for sunday, blue otherwise
  // Note: if the icon file (or path) does not exist, the icon will be created
  // with the icon for the icon editor.
  if ( DayOfWeek( ShowDate.da_day, ShowDate.da_mon, ShowDate.da_year ) == 7 )
    sprintf( dayicon, "%s\\icons\\R%d.ico",
	     path, ShowDate.da_day );
  else
    sprintf( dayicon, "%s\\icons\\N%d.ico",
	     path, ShowDate.da_day );

  // initialize the setup string for the icon object
  sprintf( setup, "OBJECTID=<CMC_CAL_ICON>;TITLE=%s;ICONFILE=%s",
	   objecttext, dayicon );

  // create or update the icon
  hIcon = WinCreateObject( "WPIcon", objecttext, setup,
			   "<WP_DESKTOP>", CO_UPDATEIFEXISTS );
  if ( hIcon == NULLHANDLE ) // something went wrong!
    SysErrorMessage( hab, "WinCreateObject", "WPCalendar" );

  return ( hIcon );
}  /* WPCalendar */

long BuildCalendar( long cur_m, long cur_y, char *string )
{
  char szLine[STRLEN], szTemp[BIGSTR];
  long day, i, start_col, test, total_days, week;

  week = WeekCount( cur_m, cur_y );

  total_days = TotalDaysOfMonth( cur_m, cur_y );

  sprintf( string, "%s %4d", GetMonth( cur_m ), cur_y );
  if ( weekstart == 0 )
  { if ( strcmp( language, "de" ) == 0 )
      strcpy (szLine, "KW  Mo Di Mi Do Fr Sa So" );
    if ( strcmp( language, "fr" ) == 0 )
      strcpy (szLine, "s.  Lu Ma Me Je Ve Sa Di" );
    if ( strcmp( language, "nl" ) == 0 )
      strcpy (szLine, "W#. Ma Di Wo Do Vr Za Zo" );
    else  strcpy (szLine, "We  Mo Tu We Th Fr Sa Su" ); }
  else
  { if ( strcmp( language, "de" ) == 0 )
      strcpy (szLine, "KW  So Mo Di Mi Do Fr Sa" );
    if ( strcmp( language, "fr" ) == 0 )
      strcpy (szLine, "s.  Di Lu Ma Me Je Ve Sa" );
    if ( strcmp( language, "nl" ) == 0 )
      strcpy (szLine, "W#. Zo Ma Di Wo Do Vr Za" );
    else  strcpy (szLine, "We  Su Mo Tu We Th Fr Sa" ); }
  sprintf( szTemp, "%s\r\n%s", string, szLine );
  strcpy( string, szTemp );

  sprintf( szTemp, "%s\r\n%02d ", string, week );
  strcpy( string, szTemp );

  start_col = DayOfWeek( 1, cur_m, cur_y ) + weekstart;
  if ( start_col == 8 ) start_col = 1;

  for( i = 1; i < start_col; i++ )
  { sprintf( szTemp, "%s --", string );
    strcpy( string, szTemp ); }

  for( day = 1; day <= total_days; day++ )
  { sprintf( szTemp, "%s %02d", string, day );
    strcpy( string, szTemp );
    test = day + start_col - 1;
    if ( ( test % 7 ) == 0 ) //( test  / 7 ) )
    { week++;
      if ( day != total_days )
      {	sprintf( szTemp, "%s\r\n%02d ", string, week );
	strcpy( string, szTemp );
      }
    } /* day if */
  } /* day for */

    day--;
    test = day + start_col - 1;
    if ( ( test % 7 ) != 0 )
    do
    { if ( ( total_days == 31 ) && ( start_col == 5 ) ) break;
      if ( ( total_days == 30 ) && ( start_col == 6 ) ) break;
      sprintf( szTemp, "%s --", string );
      strcpy( string, szTemp );
      day++;
    } while ( ( ( day + start_col - 1 ) % 7 ) != 0 );

  return( start_col );
}  /* BuildCalendar */

long DayOfWeek( long dd, long mm, long year )
{
  long cent, leap, dow, yr;
  static long days[] = { 7, 1, 2, 3, 4, 5, 6 };
  static long magic[] = { 6, 2, 3, 6, 1, 4, 6, 2, 5, 0, 3, 5 };

  cent = year / 100;
  yr = year - cent * 100;

  leap = ((( year % 4 ) == 0 ) && (( yr != 0 ) || (( cent % 4 ) == 0 )));

  dow = yr + ( yr / 4 ) + 6 - 2 * ( cent % 4 ) + magic[mm - 1] + dd;
  if ( (mm < 3 ) & !leap ) dow = dow + 1;
  dow = dow % 7;

  return days[dow];
} /* DayOfWeek */

long WeekCount( long cur_m, long cur_y )
{
  long DaysThisYear, i;

   DaysThisYear = 1;  // count days from 1.1. to cur_date
   if ( cur_m > 1 )
   { for ( i = 1; i < cur_m; i++ ) DaysThisYear += TotalDaysOfMonth( i, cur_y );
     // fixup for last week of next to last month filling entire line
     if ( weekstart == 1 )
       if ( DayOfWeek( TotalDaysOfMonth( cur_m - 1, cur_y ),
	    cur_m - 1, cur_y ) == 6 ) DaysThisYear +=  7;
   }

  return ( DaysThisYear / 7 );
}  /* WeekCount */

long TotalDaysOfMonth( long cur_m, long cur_y )
{
  switch( cur_m ) {
  case 2:  // february's special
  { long cent, leap, yr;

    cent = cur_y / 100;
    yr = cur_y - cent * 100;

    leap = ((( cur_y % 4 ) == 0 ) && (( yr != 0 ) || (( cent % 4 ) == 0 )));

    if ( leap == 1 )
      return 29;
    else
      return 28; }
  case 4:  // april
  case 6:  // june
  case 9:  // september
  case 11: // november
    return 30; }

  return 31;  // everyone else
} /* TotalDaysOfMonth */

char *GetMonth( long cur_m )
{
  static char *de[] =
  { "Januar", "Februar", "Mrz", "April", "Mai", "Juni", "Juli", "August",
    "September", "Oktober", "November", "Dezember", "Get Month ERROR" };
  static char *en[] =
  { "January", "February", "March", "April", "May", "June" ,"July", "August",
    "September", "October", "November", "December", "Get Month ERROR" };
  static char *fr[] =
  { "Janvier", "Fvrier", "Mars", "Avril", "Mai", "Juin", "Juillet", "Aot",
    "Septembre", "Octobre", "Novembre", "Dcembre", "ERREUR Get Month" };
  static char *nl[] =
  { "Januari", "Februari", "Maart", "April", "Mei", "Juni", "Juli", "Augustus",
    "September", "Oktober", "November", "December", "ERREUR Get Month" };

  if ( strcmp( language, "de" ) == 0 )
  { if (( cur_m < 1 ) || ( cur_m > 12 ))
      return ( de[12] );
    else
      return de[cur_m - 1]; }

  if ( strcmp( language, "fr" ) == 0 )
  { if (( cur_m < 1 ) || ( cur_m > 12 ))
      return ( fr[12] );
    else
      return fr[cur_m - 1]; }

  if ( strcmp( language, "nl" ) == 0 )
  { if (( cur_m < 1 ) || ( cur_m > 12 ))
      return ( nl[12] );
    else
      return nl[cur_m - 1]; }

  if (( cur_m < 1 ) || ( cur_m > 12 ))
    return ( en[12] );
  else
    return en[cur_m - 1];
} /* GetMonth */

